import { Component, inject, input, signal } from '@angular/core';
import Redemption from '../../shared/models/redemption';
import { MatCardModule } from '@angular/material/card';
import { TwitchRedemptionDropdownComponent } from "../twitch-redemption-dropdown/twitch-redemption-dropdown.component";
import { MatFormFieldModule } from '@angular/material/form-field';
import { ActionDropdownComponent } from '../../actions/action-dropdown/action-dropdown.component';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { MatTableModule } from '@angular/material/table';
import { RedemptionItemEditComponent } from '../redemption-item-edit/redemption-item-edit.component';
import { MatDialog } from '@angular/material/dialog';
import RedeemableAction from '../../shared/models/redeemable-action';
import { MatExpansionModule } from '@angular/material/expansion';
import { ScrollingModule } from '@angular/cdk/scrolling';
import { toTwitchRedemptionDict } from '../../shared/transformers/twitch-redemption.transformer';

@Component({
  selector: 'redemption-list',
  imports: [
    ActionDropdownComponent,
    MatButtonModule,
    MatCardModule,
    MatExpansionModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    MatTableModule,
    ScrollingModule,
    TwitchRedemptionDropdownComponent,
  ],
  templateUrl: './redemption-list.component.html',
  styleUrl: './redemption-list.component.scss'
})
export class RedemptionListComponent {
  readonly dialog = inject(MatDialog);
  displayedColumns: string[] = ['twitch-redemption', 'action-name', 'order', 'misc'];
  filter_redemption: string | undefined;
  filter_action_name: string | undefined;
  readonly panelOpenState = signal(false);

  _redemptions = input.required<Redemption[]>({ alias: 'redemptions' });
  twitchRedemptions = input.required({
    alias: 'twitchRedemptions',
    transform: toTwitchRedemptionDict,
  });
  //twitchRedemptions = computed<{ [id: string]: string } | null>(() => ({}));
  actions = input.required<RedeemableAction[]>();


  compare(a: Redemption, b: Redemption) {
    const dict = this.twitchRedemptions();
    if (!dict) {
      return 0;
    }
    return dict[a.redemption_id]?.title.localeCompare(dict[b.redemption_id]?.title) || a.order - b.order;
  }

  get redemptions() {
    const actionFilter = this.filter_action_name?.toString().toLowerCase();
    const redemptionFilter = this.filter_redemption?.toString().toLowerCase();
    let filtered = this._redemptions();
    if (redemptionFilter) {
      filtered = filtered.filter(r => this.twitchRedemptions()![r.redemption_id]?.title.toLowerCase().includes(redemptionFilter) || r.redemption_id == redemptionFilter);
    }
    if (actionFilter) {
      filtered = filtered.filter(r => !actionFilter || r.action_name.toLowerCase().includes(actionFilter));
    }
    return filtered;
  }

  getTwitchRedemptionNameById(id: string) {
    return this.twitchRedemptions()![id]?.title;
  }

  add(): void {
    this.openDialog({ id: '', user_id: '', redemption_id: '', action_name: '', order: 0, state: true });
  }

  openDialog(redemption: Redemption): void {
    this.dialog.open(RedemptionItemEditComponent, {
      data: { redemption: { ...redemption }, twitchRedemptions: this.twitchRedemptions(), redeemableActions: this.actions() },
      maxWidth: '100vw'
    });
  }
}